@@ -143,26 +143,146 @@ static void v4lconvert_crop_yuv420(unsigned char *src, unsigned char *dest,
   }
 }
 
+/* Ok, so this is not really cropping, but more the reverse, whatever */
+static void v4lconvert_add_border_rgbbgr24(
+  unsigned char *src, unsigned char *dest,
+  const struct v4l2_format *src_fmt, const struct v4l2_format *dest_fmt)
+{
+  int y;
+  int borderx = (dest_fmt->fmt.pix.width - src_fmt->fmt.pix.width) / 2;
+  int bordery = (dest_fmt->fmt.pix.height - src_fmt->fmt.pix.height) / 2;
+
+  for (y = 0; y < bordery; y++) {
+    memset(dest, 0, dest_fmt->fmt.pix.width * 3);
+    dest += dest_fmt->fmt.pix.bytesperline;
+  }
+
+  for (y = 0; y < src_fmt->fmt.pix.height; y++) {
+    memset(dest, 0, borderx * 3);
+    dest += borderx * 3;
+
+    memcpy(dest, src, src_fmt->fmt.pix.width * 3);
+    src += src_fmt->fmt.pix.bytesperline;
+    dest += src_fmt->fmt.pix.width * 3;
+
+    memset(dest, 0, borderx * 3);
+    dest += dest_fmt->fmt.pix.bytesperline -
+	    (borderx + src_fmt->fmt.pix.width) * 3;
+  }
+
+  for (y = 0; y < bordery; y++) {
+    memset(dest, 0, dest_fmt->fmt.pix.width * 3);
+    dest += dest_fmt->fmt.pix.bytesperline;
+  }
+}
+
+static void v4lconvert_add_border_yuv420(
+  unsigned char *src, unsigned char *dest,
+  const struct v4l2_format *src_fmt, const struct v4l2_format *dest_fmt)
+{
+  int y;
+  int borderx = (dest_fmt->fmt.pix.width - src_fmt->fmt.pix.width) / 2;
+  int bordery = (dest_fmt->fmt.pix.height - src_fmt->fmt.pix.height) / 2;
+
+  /* Y */
+  for (y = 0; y < bordery; y++) {
+    memset(dest, 16, dest_fmt->fmt.pix.width);
+    dest += dest_fmt->fmt.pix.bytesperline;
+  }
+
+  for (y = 0; y < src_fmt->fmt.pix.height; y++) {
+    memset(dest, 16, borderx);
+    dest += borderx;
+
+    memcpy(dest, src, src_fmt->fmt.pix.width);
+    src += src_fmt->fmt.pix.bytesperline;
+    dest += src_fmt->fmt.pix.width;
+
+    memset(dest, 16, borderx);
+    dest += dest_fmt->fmt.pix.bytesperline -
+	    (borderx + src_fmt->fmt.pix.width);
+  }
+
+  for (y = 0; y < bordery; y++) {
+    memset(dest, 16, dest_fmt->fmt.pix.width);
+    dest += dest_fmt->fmt.pix.bytesperline;
+  }
+
+  /* U */
+  for (y = 0; y < bordery / 2; y++) {
+    memset(dest, 128, dest_fmt->fmt.pix.width / 2);
+    dest += dest_fmt->fmt.pix.bytesperline / 2;
+  }
+
+  for (y = 0; y < src_fmt->fmt.pix.height / 2; y++) {
+    memset(dest, 128, borderx / 2);
+    dest += borderx / 2;
+
+    memcpy(dest, src, src_fmt->fmt.pix.width / 2);
+    src += src_fmt->fmt.pix.bytesperline / 2;
+    dest += src_fmt->fmt.pix.width / 2;
+
+    memset(dest, 128, borderx / 2);
+    dest += (dest_fmt->fmt.pix.bytesperline -
+	     (borderx + src_fmt->fmt.pix.width)) / 2;
+  }
+
+  for (y = 0; y < bordery / 2; y++) {
+    memset(dest, 128, dest_fmt->fmt.pix.width / 2);
+    dest += dest_fmt->fmt.pix.bytesperline / 2;
+  }
+
+  /* V */
+  for (y = 0; y < bordery / 2; y++) {
+    memset(dest, 128, dest_fmt->fmt.pix.width / 2);
+    dest += dest_fmt->fmt.pix.bytesperline / 2;
+  }
+
+  for (y = 0; y < src_fmt->fmt.pix.height / 2; y++) {
+    memset(dest, 128, borderx / 2);
+    dest += borderx / 2;
+
+    memcpy(dest, src, src_fmt->fmt.pix.width / 2);
+    src += src_fmt->fmt.pix.bytesperline / 2;
+    dest += src_fmt->fmt.pix.width / 2;
+
+    memset(dest, 128, borderx / 2);
+    dest += (dest_fmt->fmt.pix.bytesperline -
+	     (borderx + src_fmt->fmt.pix.width)) / 2;
+  }
+
+  for (y = 0; y < bordery / 2; y++) {
+    memset(dest, 128, dest_fmt->fmt.pix.width / 2);
+    dest += dest_fmt->fmt.pix.bytesperline / 2;
+  }
+}
+
 void v4lconvert_crop(unsigned char *src, unsigned char *dest,
   const struct v4l2_format *src_fmt, const struct v4l2_format *dest_fmt)
 {
   switch (dest_fmt->fmt.pix.pixelformat) {
     case V4L2_PIX_FMT_RGB24:
     case V4L2_PIX_FMT_BGR24:
-      if (src_fmt->fmt.pix.width  >= 2 * dest_fmt->fmt.pix.width &&
-	  src_fmt->fmt.pix.height >= 2 * dest_fmt->fmt.pix.height)
+      if (src_fmt->fmt.pix.width  <= dest_fmt->fmt.pix.width &&
+	  src_fmt->fmt.pix.height <= dest_fmt->fmt.pix.height)
+	v4lconvert_add_border_rgbbgr24(src, dest, src_fmt, dest_fmt);
+      else if (src_fmt->fmt.pix.width  >= 2 * dest_fmt->fmt.pix.width &&
+	       src_fmt->fmt.pix.height >= 2 * dest_fmt->fmt.pix.height)
 	v4lconvert_reduceandcrop_rgbbgr24(src, dest, src_fmt, dest_fmt);
       else
 	v4lconvert_crop_rgbbgr24(src, dest, src_fmt, dest_fmt);
       break;
+
     case V4L2_PIX_FMT_YUV420:
     case V4L2_PIX_FMT_YVU420:
-      if (src_fmt->fmt.pix.width  >= 2 * dest_fmt->fmt.pix.width &&
-	  src_fmt->fmt.pix.height >= 2 * dest_fmt->fmt.pix.height)
+      if (src_fmt->fmt.pix.width  <= dest_fmt->fmt.pix.width &&
+	  src_fmt->fmt.pix.height <= dest_fmt->fmt.pix.height)
+	v4lconvert_add_border_yuv420(src, dest, src_fmt, dest_fmt);
+      else if (src_fmt->fmt.pix.width  >= 2 * dest_fmt->fmt.pix.width &&
+	       src_fmt->fmt.pix.height >= 2 * dest_fmt->fmt.pix.height)
 	v4lconvert_reduceandcrop_yuv420(src, dest, src_fmt, dest_fmt);
       else
 	v4lconvert_crop_yuv420(src, dest, src_fmt, dest_fmt);
-
       break;
   }
 }
